@@ -1,6 +1,6 @@
 /* collection of junk waiting time to sort out
-   Copyright (C) 1998, 1999, 2000, 2001, 2003, 2007, 2008, 2009, 2010, 2011
-   Free Software Foundation, Inc.
+   Copyright (C) 1998-2001, 2003, 2007-2012 Free Software Foundation,
+   Inc.
    Contributed by Red Hat
 
 This file is part of the GNU Simulators.
